@@ -177,6 +177,13 @@ class Tool(object):
                 Terminating newline may be omitted to produce output from multiple
                 invoked action on a single line.
 
+        @classmethod
+        __format_help__()
+            output: format help string
+                
+                Implement to provide output formatting specific help for this
+                tool.
+
         __timeout__()
             output: str or None
 
@@ -407,6 +414,14 @@ class Tool(object):
     def __timeout__(self):
         return None
 
+    #
+    # __format_help__ prints output format specific help
+    #
+    @classmethod
+    def __format_help__(cls):
+        return 'Not implemented'
+
+
 
 class Test1(unittest.TestCase):
     def test_file(self):
