@@ -17,6 +17,7 @@ from xnet.net.ipv4 import IPRangeIterator
 class IISShortnameTool(WebgetTool):
 
     __description__ = 'identify IIS shortname functionality'
+    __toolname__ = 'iissn'
     __itemname__ = 'url'
 
     cmdline_options = [
@@ -50,7 +51,7 @@ class IISShortnameTool(WebgetTool):
         if not url.endswith('/'):
             url += '/'
         url1 = url + '*~1*/.aspx'
-        url2 = url + 'poipoi*~1*/.aspx'
+        url2 = url + 'poipoipoi*~1*/.aspx'
         result['url'] = url
         result['url1'] = url1
         result['url2'] = url2
@@ -113,6 +114,26 @@ class IISShortnameTool(WebgetTool):
             )
         return output
 
+    @classmethod
+    def __format_help__(cls):
+        name = cls.__toolname__
+        output = '''
+Format variables for %s:
+
+ url         - url to probe
+ host        - Host header
+ vulnerable  - boolean result
+ code1       - response code from url+'*~1*/.aspx'
+ code2       - response code from url+'poipoipoi*~1*/.aspx'
+
+Default format:
+ '{url} {host} {vulnerable} {code1} {code2}'
+
+Default format if verbose (-v):
+ '{url} {vulnerable}'
+        ''' % cls.__toolname__
+        return output
+
 
 def main():
     import xnet.tools
@@ -120,5 +141,6 @@ def main():
 
 
 if __name__ == "__main__":
-    doctest.testmod()
-    unittest.main()
+    main()
+    #doctest.testmod()
+    #unittest.main()
