@@ -159,7 +159,7 @@ def _get_output_filepath(a, base):
     item = str(a._item)
     item = item.replace('/', '__')
     item = item.replace('\\', '--')
-    path = base + '_' + item
+    path = base + item
     return path
 
 def run(ToolImplementation, stdin_data_arg=None, tool_reads_stdin=False):
@@ -232,16 +232,25 @@ def _run(ToolImplementation, stdin_data_arg, tool_reads_stdin):
         print ToolImplementation.__format_help__()
         sys.exit(0)
 
+    if opt.split_tee:
+        opt.split_output = opt.split_tee
+
     pool = gevent.pool.Pool(_parallelism)
     actions = []  # all spawned actions
     greenlets = []
 
-    waitkill_greenlets = []
+    wkpool = None
+    if _wait:
+        wkpool = gevent.pool.Pool(_parallelism)
 
-    def waitkill(g):
-        gevent.sleep(_wait)
+    def waitkill(g, killtime):
+        sleeptime = killtime - time.time()
+        #print 'waitkill', sleeptime
+        if sleeptime > 0:
+            gevent.sleep(sleeptime)
         if not g.ready():
             gevent.kill(g)
+            g.join()
 
     inputchain = ToolImplementation.__massage__(inputchain)
 
@@ -252,91 +261,98 @@ def _run(ToolImplementation, stdin_data_arg, tool_reads_stdin):
     if tool_reads_stdin:
         pool.spawn(stdin_disperser_greenlet, pool)
 
-    #
-    # FIXME: --wait only activated after all greenlets
-    # are spawned, so inputs larger than poolsize
-    # are unaffected.
-    # FIXME: output is only generated when all greenlets
-    # are done; should be as real time as possible in order
-    #
+
     _t = 0.0
+    killtime = 0
     for (i, line) in enumerate(inputchain):
+        action = ToolImplementation(opt, **kwargs)
         pool.wait_available()
-        a = ToolImplementation(opt, **kwargs)
-        actions.append(a)
+        if _wait:
+            killtime = time.time() + _wait
+        g = pool.spawn(action, line, inputchain)
+        g.action = action
+        greenlets.append(g)
+        if _wait:
+            wkpool.wait_available()
+            wk = wkpool.spawn(waitkill, g, killtime)
+
         #
-        # handle interval and spawn
+        # handle finished actions
+        #
+        while len(greenlets) and greenlets[0].ready():
+            action = greenlets[0].action
+            output_action_result(action, opt, _outfile)
+            del action
+            del greenlets[0]
+
+        #
+        # handle interval
         #
         _this_interval = _interval - (time.time() - _t)
         if _this_interval > 0:
             gevent.sleep(_this_interval)
-        g = pool.spawn(a, line, inputchain)
-
-        if not _wait is None:
-            wk = gevent.spawn(waitkill, g)
-            waitkill_greenlets.append(wk)
-            if len(waitkill_greenlets) > 2 * _parallelism:
-                finished, waitkill_greenlets = waitkill_greenlets[:_parallelism],\
-                    waitkill_greenlets[_parallelism:]
-                t = time.time()
-                gevent.joinall(waitkill_greenlets)
-                tdiff = time.time() - t
-                print 'waitkill jointime:', tdiff
-
         _t = time.time()
-        greenlets.append(g)
 
-    timeout = _wait
-    if not timeout is None:
-        timeout += 1  # one second of grace time for greenlets to exit
-    gevent.joinall(greenlets, timeout=timeout)
+
+    if wkpool:
+        wkpool.join()
+    gevent.joinall(greenlets) #, timeout=timeout)
     not_done = filter(lambda g: (not g.ready()), greenlets)
     if len(not_done) > 0 and not _wait is None:
-        print 'not_done has contents inspite of _wait and grace time'
+        print 'ERROR: not_done has contents inspite of _wait and grace time'
         print not_done
+
+    #
+    # Force-kill greenlets that didn't die in spite of 1 sec of grace time.
+    #
     gevent.killall(not_done, block=True)
     gevent.joinall(not_done)
 
     #
-    # also assert all waitkill_greenlets are done
+    # print results
     #
+    for g in greenlets:
+        action = g.action
+        output_action_result(action, opt, _outfile)
+        del action
 
-    if opt.split_tee:
-        opt.split_output = opt.split_tee
 
-    for a in actions:
-        outfile = _outfile
-        if opt.split_output:
-            path = _get_output_filepath(a, opt.split_output)
-            outfile = open(path, 'wb')
-
-        if not a.error is None:
-            print_error(a.error)
-        elif not a.finished:
-            if not opt.supress_timeout:
-                msg = a.__timeout__()
-                if a.stderr:
-                    outfile = sys.stderr
-                if not msg is None:
-                    outfile.write(msg)
-                    if opt.split_tee and not a.stderr:
-                        _outfile.write(msg)
-        else:
-            msg = str(a)
-            #
-            # ignore empty output from Tool.__format__()
-            #
-            if len(msg) == 0:
-                continue
-            msg += '\n'
+def output_action_result(a, opt, _outfile):
+    outfile = _outfile
+    if opt.split_output:
+        path = _get_output_filepath(a, opt.split_output)
+        outfile = open(path, 'wb')
+    if not a.error is None:
+        print_error(a.error)
+    elif not a.finished:
+        if not opt.supress_timeout:
+            msg = a.__timeout__()
             if a.stderr:
                 outfile = sys.stderr
-            outfile.write(msg)
-            if opt.split_tee and not a.stderr:
-                _outfile.write(msg)
+            if not msg is None:
+                outfile.write(msg)
+                if opt.split_tee and not a.stderr:
+                    _outfile.write(msg)
+    else:
+        msg = str(a)
+        #
+        # ignore empty output from Tool.__format__()
+        #
+        if len(msg) == 0:
+            return
+        msg += '\n'
+        if a.stderr:
+            outfile = sys.stderr
+        outfile.write(msg)
+        if opt.split_tee and not a.stderr:
+            _outfile.write(msg)
 
-        if opt.split_output:
-            outfile.close()
+    _outfile.flush()
+    if not outfile is _outfile:
+        outfile.flush()
+
+    if opt.split_output:
+        outfile.close()
 
 
 if __name__ == "__main__":
