@@ -19,6 +19,8 @@ import gevent
 from xnet.tools import Tool
 from xnet.net.ipv4 import IPRangeIterator
 
+from xnet.debug import pdb
+
 
 class SSLInfoException(Exception):
     pass
@@ -26,8 +28,9 @@ class SSLInfoException(Exception):
 
 class SSLInfoTool(Tool):
 
-    __description__ = 'get ssl and certificate information'
+    __toolname__ = 'sslinfo'
     __itemname__ = 'host'
+    __description__ = 'get ssl and certificate information'
 
     cmdline_options = [
         ('-p', '--port', 'tcp port',
@@ -71,7 +74,6 @@ class SSLInfoTool(Tool):
             cert = ssl.get_server_certificate(addr)
             cert = self._fix_cert(cert)
             x509 = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, cert)
-            components = dict(x509.get_subject().get_components())
         except socket.error, e:
             #
             # errno numbers differ between platforms,
@@ -93,9 +95,24 @@ class SSLInfoTool(Tool):
             self.set_error(errmsg)
             self.stderr = True
             return None
+        result['host'] = addr[0]
+        result['port'] = addr[1]
         result['cert'] = cert
         result['x509'] = x509
-        result['components'] = dict(components)
+        result['notbefore'] = x509.get_notBefore()
+        result['notafter'] = x509.get_notAfter()
+        result['serial'] = str(x509.get_serial_number()).replace('L', '')
+        issuer = x509.get_issuer().get_components()
+        result['issuer'] = ''.join('/' + n + '=' + v for (n, v) in issuer)
+        subject = x509.get_subject().get_components()
+        result['subject'] = ''.join('/' + n + '=' + v for (n, v) in subject)
+        subject = dict(subject)
+        result['C'] = subject.get('C', '-')
+        result['ST'] = subject.get('ST', '-')
+        result['O'] = subject.get('O', '-')
+        result['OU'] = subject.get('OU', '-')
+        result['CN'] = subject.get('CN', '-')
+        result['email'] = subject.get('emailAddress', '-')
         return result
 
     def _fix_cert(self, cert):
@@ -111,11 +128,11 @@ class SSLInfoTool(Tool):
     def __format__(self, line, parse_result, action_result):
         output = ''
         host, port = parse_result['addr']
-        components = action_result['components']
+        cd = action_result['components_dict']
         if self.options.cn:
-            output = '{0} {1}'.format(host, components.get('CN', None))
+            output = '{0} {1}'.format(host, cd.get('CN', None))
         else:
-            for (name, val) in components.iteritems():
+            for (name, val) in cd.iteritems():
                 output += '{0} {1}={2}\n'.format(host, name, val)
             if output[-1] == '\n':
                 output = output[:-1]
@@ -124,6 +141,27 @@ class SSLInfoTool(Tool):
     def __timeout__(self):
         return 'timeout {0} {1}\n'.format(*self._addr)
 
+    @classmethod
+    def __format_help__(cls):
+        name = cls.__toolname__
+        output = '''
+Format variables available for %s.
+(default format is complex and not reproducable from commandline)
+
+ notbefore   - not valid before
+ notafter    - not valid after
+ serial      - cert serial number
+ issuer      - issuer common name, organizational unit, ...
+ subject     - subject common name, organizational unit, ...
+ C           - subject country
+ ST          - subject state
+ O           - subject organization
+ OU          - subject organizational unit
+ CN          - subject common name
+ email       - subject email address
+        ''' % cls.__toolname__
+        return output
+
 
 def main():
     import xnet.tools
